@@ -18,8 +18,7 @@
 
    You should have received a copy of the GNU Library General Public
    License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   see <http://www.gnu.org/licenses/>.
 
    Authors: John Sullivan <sullivan@eazel.com>
             Ramiro Estrugo <ramiro@eazel.com>
@@ -330,7 +329,7 @@ eel_gtk_message_dialog_set_details_label (GtkMessageDialog *dialog,
 	label = gtk_label_new (details_text);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 	gtk_label_set_selectable (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+        gtk_label_set_xalign (GTK_LABEL (label), 0);
 
 	gtk_container_add (GTK_CONTAINER (expander), label);
 	gtk_box_pack_start (GTK_BOX (content_area), expander, FALSE, FALSE, 0);
