@@ -1096,7 +1096,7 @@ trash_retrieve_files_to_restore_thread (GSimpleAsyncResult *res,
 	GFile *trash;
 	GError *error = NULL;
 
-	to_restore = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal, 
+	to_restore = g_hash_table_new_full (g_file_hash, (GEqualFunc) g_file_equal,
 					    g_object_unref, g_object_unref);
 
 	trash = g_file_new_for_uri ("trash:///");
@@ -1218,9 +1218,6 @@ trash_undo_func (NautilusFileUndoInfo *info,
 {
 	NautilusFileUndoInfoTrash *self = NAUTILUS_FILE_UNDO_INFO_TRASH (info);
 
-	/* Internally managed op, pop flag. */
-	nautilus_file_undo_manager_pop_flag ();
-
 	trash_retrieve_files_to_restore_async (self, trash_retrieve_files_ready, NULL);
 }
 
@@ -1349,10 +1346,6 @@ rec_permissions_undo_func (NautilusFileUndoInfo *info,
 {
 	NautilusFileUndoInfoRecPermissions *self = NAUTILUS_FILE_UNDO_INFO_REC_PERMISSIONS (info);
 
-	/* Internally managed op, pop flag. */
-	/* TODO: why? */
-	nautilus_file_undo_manager_pop_flag ();
-
 	if (g_hash_table_size (self->priv->original_permissions) > 0) {
 		GList *gfiles_list;
 		guint32 perm;
