@@ -256,7 +256,6 @@ search_add_volumes_and_bookmarks (PendingSearch *search)
   NautilusSearchHit *hit;
   NautilusBookmark *bookmark;
   const gchar *name;
-  gint length, idx;
   gchar *string, *uri;
   gdouble match;
   GList *l, *m, *drives, *volumes, *mounts, *mounts_to_check, *candidates;
@@ -266,17 +265,17 @@ search_add_volumes_and_bookmarks (PendingSearch *search)
   GFile *location;
   SearchHitCandidate *candidate;
   NautilusBookmarkList *bookmarks;
+  GList *all_bookmarks;
   GVolumeMonitor *volume_monitor;
 
   bookmarks = nautilus_application_get_bookmarks (NAUTILUS_APPLICATION (g_application_get_default ()));
+  all_bookmarks = nautilus_bookmark_list_get_all (bookmarks);
   volume_monitor = g_volume_monitor_get ();
   candidates = NULL;
 
   /* first add bookmarks */
-  length = nautilus_bookmark_list_length (bookmarks);
-  for (idx = 0; idx < length; idx++) {
-    bookmark = nautilus_bookmark_list_item_at (bookmarks, idx);
-
+  for (l = all_bookmarks; l != NULL; l = l->next) {
+    bookmark = NAUTILUS_BOOKMARK (l->data);
     name = nautilus_bookmark_get_name (bookmark);
     if (name == NULL)
       continue;
@@ -393,9 +392,10 @@ execute_search (NautilusShellSearchProvider *self,
                 GDBusMethodInvocation          *invocation,
                 gchar                         **terms)
 {
-  gchar *terms_joined, *home_uri;
+  gchar *terms_joined;
   NautilusQuery *query;
   PendingSearch *pending_search;
+  GFile *home;
 
   cancel_current_search (self);
 
@@ -407,12 +407,12 @@ execute_search (NautilusShellSearchProvider *self,
   }
 
   terms_joined = g_strjoinv (" ", terms);
-  home_uri = nautilus_get_home_directory_uri ();
+  home = g_file_new_for_path (g_get_home_dir ());
 
   query = nautilus_query_new ();
   nautilus_query_set_show_hidden_files (query, FALSE);
   nautilus_query_set_text (query, terms_joined);
-  nautilus_query_set_location (query, home_uri);
+  nautilus_query_set_location (query, home);
 
   pending_search = g_slice_new0 (PendingSearch);
   pending_search->invocation = g_object_ref (invocation);
@@ -440,7 +440,7 @@ execute_search (NautilusShellSearchProvider *self,
                                       query);
   nautilus_search_provider_start (NAUTILUS_SEARCH_PROVIDER (pending_search->engine));
 
-  g_free (home_uri);
+  g_clear_object (&home);
   g_free (terms_joined);
 }
 
