@@ -1,4 +1,4 @@
-/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-
+/* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 8; tab-width: 8 -*-
 
    nautilus-window-slot.c: Nautilus window slot
  
@@ -100,7 +100,6 @@ struct NautilusWindowSlotDetails {
 
 	/* Query editor */
 	NautilusQueryEditor *query_editor;
-	GtkWidget *query_editor_revealer;
 	gulong qe_changed_id;
 	gulong qe_cancel_id;
 	gulong qe_activated_id;
@@ -123,8 +122,8 @@ struct NautilusWindowSlotDetails {
 	GCancellable *mount_cancellable;
 	GError *mount_error;
 	gboolean tried_mount;
-        gchar *view_mode_before_search;
-        gchar *view_mode_before_places;
+        gint view_mode_before_search;
+        gint view_mode_before_places;
 };
 
 static guint signals[LAST_SIGNAL] = { 0 };
@@ -136,9 +135,9 @@ static void hide_query_editor (NautilusWindowSlot *slot);
 static void nautilus_window_slot_sync_actions (NautilusWindowSlot *slot);
 static void nautilus_window_slot_connect_new_content_view (NautilusWindowSlot *slot);
 static void nautilus_window_slot_disconnect_content_view (NautilusWindowSlot *slot);
-static gboolean nautilus_window_slot_content_view_matches (NautilusWindowSlot *slot, const char *iid);
+static gboolean nautilus_window_slot_content_view_matches (NautilusWindowSlot *slot, guint id);
 static NautilusView* nautilus_window_slot_get_view_for_location (NautilusWindowSlot *slot, GFile *location);
-static void nautilus_window_slot_set_content_view (NautilusWindowSlot *slot, const char	*id);
+static void nautilus_window_slot_set_content_view (NautilusWindowSlot *slot, guint id);
 static void nautilus_window_slot_set_loading (NautilusWindowSlot *slot, gboolean loading);
 char * nautilus_window_slot_get_location_uri (NautilusWindowSlot *slot);
 static void nautilus_window_slot_set_search_visible (NautilusWindowSlot *slot,
@@ -164,56 +163,47 @@ nautilus_window_slot_get_view_for_location (NautilusWindowSlot *slot,
 	 * for the desktop window.
 	 */
         if (NAUTILUS_IS_DESKTOP_WINDOW (window)) {
-                view = NAUTILUS_VIEW (nautilus_files_view_new (NAUTILUS_DESKTOP_VIEW_ID, slot));
+                view = NAUTILUS_VIEW (nautilus_files_view_new (NAUTILUS_VIEW_DESKTOP_ID, slot));
         } else if (nautilus_file_is_other_locations (file)) {
                 view = NAUTILUS_VIEW (nautilus_places_view_new ());
 
                 /* Save the current view, so we can go back after places view */
                 if (slot->details->content_view && NAUTILUS_IS_FILES_VIEW (slot->details->content_view)) {
-                        g_clear_pointer (&slot->details->view_mode_before_places, g_free);
-                        slot->details->view_mode_before_places = g_strdup (nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (slot->details->content_view)));
+                        slot->details->view_mode_before_places = nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (slot->details->content_view));
                 }
         } else {
-                gchar *view_id;
+                guint view_id;
 
-                view_id = NULL;
+                view_id = NAUTILUS_VIEW_INVALID_ID;
 
-                /* If we are in search, try to use by default list view. This will be deactivated
-                 * if the user manually switch to a diferent view mode */
+                /* If we are in search, try to use by default list view. */
                 if (nautilus_file_is_in_search (file)) {
-                        if (g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_LIST_VIEW_ON_SEARCH)) {
-                                /* If it's already set, is because we already made the change to search mode,
-                                 * so the view mode of the current view will be the one search is using,
-                                 * which is not the one we are interested in */
-                                if (slot->details->view_mode_before_search == NULL) {
-                                        slot->details->view_mode_before_search = g_strdup (nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (slot->details->content_view)));
-                                }
-                                view_id = g_strdup (NAUTILUS_LIST_VIEW_ID);
-                        } else {
-                                g_free (slot->details->view_mode_before_search);
-                                slot->details->view_mode_before_search = NULL;
+                        /* If it's already set, is because we already made the change to search mode,
+                         * so the view mode of the current view will be the one search is using,
+                         * which is not the one we are interested in */
+                        if (slot->details->view_mode_before_search == NAUTILUS_VIEW_INVALID_ID) {
+                                slot->details->view_mode_before_search = nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (slot->details->content_view));
                         }
-                }
-
-                /* If there is already a view, just use the view mode that it's currently using, or
-                 * if we were on search before, use what we were using before entering
-                 * search mode */
-	        if (slot->details->content_view != NULL && view_id == NULL) {
-                        if (slot->details->view_mode_before_search != NULL) {
+                        view_id = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_SEARCH_VIEW);
+                } else if (slot->details->content_view != NULL) {
+                        /* If there is already a view, just use the view mode that it's currently using, or
+                         * if we were on search before, use what we were using before entering
+                         * search mode */
+                        if (slot->details->view_mode_before_search != NAUTILUS_VIEW_INVALID_ID) {
                                 view_id = slot->details->view_mode_before_search;
-                                slot->details->view_mode_before_search = NULL;
+                                slot->details->view_mode_before_search = NAUTILUS_VIEW_INVALID_ID;
                         } else if (NAUTILUS_IS_PLACES_VIEW (slot->details->content_view)) {
                                 view_id = slot->details->view_mode_before_places;
-                                slot->details->view_mode_before_places = NULL;
+                                slot->details->view_mode_before_places = NAUTILUS_VIEW_INVALID_ID;
                         } else {
-		                view_id = g_strdup (nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (slot->details->content_view)));
+		                view_id = nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (slot->details->content_view));
                         }
 	        }
 
                 /* If there is not previous view in this slot, use the default view mode
                  * from preferences */
-	        if (view_id == NULL) {
-		        view_id = nautilus_global_preferences_get_default_folder_viewer_preference_as_iid ();
+	        if (view_id == NAUTILUS_VIEW_INVALID_ID) {
+		        view_id = g_settings_get_enum (nautilus_preferences, NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER);
 	        }
 
                 /* Try to reuse the current view */
@@ -222,8 +212,6 @@ nautilus_window_slot_get_view_for_location (NautilusWindowSlot *slot,
                 } else {
                         view = NAUTILUS_VIEW (nautilus_files_view_new (view_id, slot));
                 }
-
-                g_free (view_id);
         }
 
         nautilus_file_unref (file);
@@ -233,30 +221,28 @@ nautilus_window_slot_get_view_for_location (NautilusWindowSlot *slot,
 
 static gboolean
 nautilus_window_slot_content_view_matches (NautilusWindowSlot *slot,
-                                           const char         *iid)
+                                           guint                id)
 {
 	if (slot->details->content_view == NULL) {
 		return FALSE;
 	}
 
-        if (!iid && NAUTILUS_IS_PLACES_VIEW (slot->details->content_view)) {
+        if (id == NAUTILUS_VIEW_INVALID_ID && NAUTILUS_IS_PLACES_VIEW (slot->details->content_view)) {
                 return TRUE;
-        } else if (iid && NAUTILUS_IS_FILES_VIEW (slot->details->content_view)){
-                return g_strcmp0 (nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (slot->details->content_view)), iid) == 0;
+        } else if (id != NAUTILUS_VIEW_INVALID_ID && NAUTILUS_IS_FILES_VIEW (slot->details->content_view)){
+                return nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (slot->details->content_view)) == id;
         } else {
                 return FALSE;
         }
 }
 
 static void
-check_search_visible (NautilusWindowSlot *slot)
+update_search_visible (NautilusWindowSlot *slot)
 {
         NautilusQuery *query;
         NautilusView *view;
-        gchar *text;
         GAction *action;
 
-        query = nautilus_query_editor_get_query (slot->details->query_editor);
         action =  g_action_map_lookup_action (G_ACTION_MAP (slot->details->slot_action_group),
                                               "search-visible");
         /* Don't allow search on desktop */
@@ -265,18 +251,21 @@ check_search_visible (NautilusWindowSlot *slot)
 
         view = nautilus_window_slot_get_current_view (slot);
         /* If we changed location just to another search location, for example,
-         * when changing the query, just keep the search visible */
-        if (nautilus_view_is_searching (view))
+         * when changing the query, just keep the search visible.
+         * Make sure the search is visible though, since we could be returning
+         * from a previous search location when using the history */
+        if (nautilus_view_is_searching (view)) {
+                nautilus_window_slot_set_search_visible (slot, TRUE);
                 return;
+         }
 
+        query = nautilus_query_editor_get_query (slot->details->query_editor);
         if (query) {
-                text = nautilus_query_get_text (query);
                 /* If the view is not searching, but search is visible, and the
                  * query is empty, we don't hide it. Some users enable the search
                  * and then change locations, then they search. */
-                if (strlen (text) != 0)
+                 if (!nautilus_query_is_empty (query))
                         nautilus_window_slot_set_search_visible (slot, FALSE);
-                g_free (text);
                 g_object_unref (query);
         }
 }
@@ -295,16 +284,16 @@ nautilus_window_slot_sync_actions (NautilusWindowSlot *slot)
 		return;
 	}
 
-        /* Check if we need to close the search or not after changing the location.
+        /* Check if we need to close the search or show search after changing the location.
          * Needs to be done after the change has been done, if not, a loop happens,
          * because setting the search enabled or not actually opens a location */
-        check_search_visible (slot);
+        update_search_visible (slot);
 
         /* Files view mode */
         action =  g_action_map_lookup_action (G_ACTION_MAP (slot->details->slot_action_group), "files-view-mode");
         if (NAUTILUS_IS_FILES_VIEW (nautilus_window_slot_get_current_view (slot)) &&
             !NAUTILUS_IS_DESKTOP_CANVAS_VIEW (nautilus_window_slot_get_current_view (slot))) {
-                variant = g_variant_new_string (nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (nautilus_window_slot_get_current_view (slot))));
+                variant = g_variant_new_uint32 (nautilus_files_view_get_view_id (NAUTILUS_FILES_VIEW (nautilus_window_slot_get_current_view (slot))));
                 g_simple_action_set_enabled (G_SIMPLE_ACTION (action), TRUE);
                 g_action_change_state (action, variant);
         } else {
@@ -353,8 +342,7 @@ hide_query_editor (NautilusWindowSlot *slot)
 
         view = nautilus_window_slot_get_current_view (slot);
 
-	gtk_revealer_set_reveal_child (GTK_REVEALER (slot->details->query_editor_revealer),
-				       FALSE);
+	gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (slot->details->query_editor), FALSE);
 
 	if (slot->details->qe_changed_id > 0) {
 		g_signal_handler_disconnect (slot->details->query_editor, slot->details->qe_changed_id);
@@ -408,11 +396,9 @@ static void
 show_query_editor (NautilusWindowSlot *slot)
 {
         NautilusView *view;
-        GFile *location;
 
         view = nautilus_window_slot_get_current_view (slot);
 
-        location = nautilus_window_slot_get_current_location (slot);
         if (nautilus_view_is_searching (view)) {
                 NautilusQuery *query;
 
@@ -421,13 +407,9 @@ show_query_editor (NautilusWindowSlot *slot)
                 if (query != NULL) {
                         nautilus_query_editor_set_query (slot->details->query_editor, query);
                 }
-        } else {
-                /* In this way, when the query changes it will open the actual search */
-                nautilus_query_editor_set_location (slot->details->query_editor, location);
         }
 
-	gtk_revealer_set_reveal_child (GTK_REVEALER (slot->details->query_editor_revealer),
-				       TRUE);
+        gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (slot->details->query_editor), TRUE);
 	gtk_widget_grab_focus (GTK_WIDGET (slot->details->query_editor));
 
 	if (slot->details->qe_changed_id == 0) {
@@ -485,7 +467,8 @@ nautilus_window_slot_handle_event (NautilusWindowSlot *slot,
 	retval = FALSE;
 	window = nautilus_window_slot_get_window (slot);
 	if (!NAUTILUS_IS_DESKTOP_WINDOW (window)) {
-		retval = nautilus_query_editor_handle_event (slot->details->query_editor, event);
+                retval = gtk_search_bar_handle_event (GTK_SEARCH_BAR (slot->details->query_editor),
+                                                      (GdkEvent*) event);
 	}
 
 	if (retval) {
@@ -536,7 +519,7 @@ remove_all_extra_location_widgets (GtkWidget *widget,
 	NautilusDirectory *directory;
 
 	directory = nautilus_directory_get (slot->details->location);
-	if (widget != GTK_WIDGET (slot->details->query_editor_revealer)) {
+	if (widget != GTK_WIDGET (slot->details->query_editor)) {
 		gtk_container_remove (GTK_CONTAINER (slot->details->extra_location_widgets), widget);
 	}
 
@@ -635,11 +618,12 @@ nautilus_window_slot_constructed (GObject *object)
 	gtk_widget_show (extras_vbox);
 
 	slot->details->query_editor = NAUTILUS_QUERY_EDITOR (nautilus_query_editor_new ());
-	slot->details->query_editor_revealer = gtk_revealer_new ();
-	gtk_container_add (GTK_CONTAINER (slot->details->query_editor_revealer),
-			   GTK_WIDGET (slot->details->query_editor));
-	gtk_widget_show_all (slot->details->query_editor_revealer);
-	nautilus_window_slot_add_extra_location_widget (slot, slot->details->query_editor_revealer);
+        gtk_widget_show (GTK_WIDGET (slot->details->query_editor));
+        nautilus_window_slot_add_extra_location_widget (slot, GTK_WIDGET (slot->details->query_editor));
+
+        g_object_bind_property (slot, "location",
+                                slot->details->query_editor, "location",
+                                G_BINDING_DEFAULT);
 
 	slot->details->title = g_strdup (_("Loading…"));
 }
@@ -672,45 +656,29 @@ action_files_view_mode (GSimpleAction *action,
 		        GVariant      *value,
 		        gpointer       user_data)
 {
-        const gchar *name;
         NautilusWindowSlot *slot;
+        const gchar *preferences_key;
+        guint view_id;
 
-        name =  g_variant_get_string (value, NULL);
+        view_id =  g_variant_get_uint32 (value);
         slot = NAUTILUS_WINDOW_SLOT (user_data);
 
         if (!NAUTILUS_IS_FILES_VIEW (nautilus_window_slot_get_current_view (slot)))
-          return;
+                return;
 
-        nautilus_window_slot_set_content_view (slot, name);
-        if (g_strcmp0 (name, "list") == 0) {
-                    /* If this change is caused because of the automatic list view
-                     * switch for search, don't set as default list view */
-                    if (!(nautilus_view_is_searching (nautilus_window_slot_get_current_view (slot))  &&
-                          g_settings_get_boolean (nautilus_preferences, NAUTILUS_PREFERENCES_LIST_VIEW_ON_SEARCH))) {
-                            g_settings_set_enum (nautilus_preferences,
-                                                 NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER,
-                                                 NAUTILUS_DEFAULT_FOLDER_VIEWER_LIST_VIEW);
-                    }
-        } else if (g_strcmp0 (name, "grid") == 0) {
-                    /* If the user manually changed the view mode to grid, disable the automatic
-                     * switch to list view on search */
-                    if (nautilus_view_is_searching (nautilus_window_slot_get_current_view (slot))) {
-                          g_settings_set_boolean (nautilus_preferences,
-                                                  NAUTILUS_PREFERENCES_LIST_VIEW_ON_SEARCH,
-                                                  FALSE);
-                    }
-                    g_settings_set_enum (nautilus_preferences,
-                                         NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER,
-                                         NAUTILUS_DEFAULT_FOLDER_VIEWER_ICON_VIEW);
-        } else {
-                g_assert_not_reached ();
-        }
+        nautilus_window_slot_set_content_view (slot, view_id);
+        preferences_key = nautilus_view_is_searching (nautilus_window_slot_get_current_view (slot)) ?
+                          NAUTILUS_PREFERENCES_SEARCH_VIEW :
+                          NAUTILUS_PREFERENCES_DEFAULT_FOLDER_VIEWER;
+
+        g_settings_set_enum (nautilus_preferences, preferences_key, view_id);
 
         g_simple_action_set_state (action, value);
 }
 
 const GActionEntry slot_entries[] = {
-        { "files-view-mode", NULL, "s", "''", action_files_view_mode },
+        /* 4 is NAUTILUS_VIEW_INVALID_ID */
+        { "files-view-mode", NULL, "u", "uint32 4", action_files_view_mode },
         { "search-visible", NULL, NULL, "false", action_search_visible },
 };
 
@@ -732,9 +700,11 @@ nautilus_window_slot_init (NautilusWindowSlot *slot)
         gtk_widget_insert_action_group (GTK_WIDGET (slot),
                                         "slot",
                                         G_ACTION_GROUP (slot->details->slot_action_group));
-        nautilus_application_add_accelerator (app, "slot.files-view-mode('list')", "<control>1");
-        nautilus_application_add_accelerator (app, "slot.files-view-mode('grid')", "<control>2");
+        nautilus_application_add_accelerator (app, "slot.files-view-mode(1)", "<control>1");
+        nautilus_application_add_accelerator (app, "slot.files-view-mode(0)", "<control>2");
         nautilus_application_add_accelerator (app, "slot.search-visible", "<control>f");
+
+        slot->details->view_mode_before_search = NAUTILUS_VIEW_INVALID_ID;
 }
 
 #define DEBUG_FLAG NAUTILUS_DEBUG_WINDOW
@@ -755,7 +725,6 @@ static void begin_location_change                     (NautilusWindowSlot
                                                        const char                 *scroll_pos);
 static void free_location_change                      (NautilusWindowSlot         *slot);
 static void end_location_change                       (NautilusWindowSlot         *slot);
-static void cancel_location_change                    (NautilusWindowSlot         *slot);
 static void got_file_info_for_view_selection_callback (NautilusFile               *file,
 						       gpointer                    callback_data);
 static gboolean setup_view                            (NautilusWindowSlot *slot,
@@ -846,8 +815,6 @@ check_force_reload (GFile                      *location,
 
 	if (type == NAUTILUS_LOCATION_CHANGE_RELOAD) {
 		force_reload = TRUE;
-	} else if (!nautilus_monitor_active ()) {
-		force_reload = TRUE;
 	} else {
 		force_reload = !nautilus_directory_is_local (directory);
 	}
@@ -915,11 +882,7 @@ begin_location_change (NautilusWindowSlot         *slot,
         nautilus_window_slot_disconnect_content_view (slot);
         /* We are going to change the location, so make sure we stop any loading
          * or searching of the previous view, so we avoid to be slow */
-        if (slot->details->content_view && NAUTILUS_IS_FILES_VIEW (slot->details->content_view)) {
-                nautilus_files_view_stop_loading (NAUTILUS_FILES_VIEW (slot->details->content_view));
-        }
-
-	end_location_change (slot);
+        nautilus_window_slot_stop_loading (slot);
 
 	nautilus_window_slot_set_allow_stop (slot, TRUE);
 
@@ -976,8 +939,6 @@ nautilus_window_slot_set_location (NautilusWindowSlot *slot,
 
 	nautilus_window_slot_update_title (slot);
 
-        nautilus_query_editor_set_location (slot->details->query_editor, location);
-
 	if (old_location) {
 		g_object_unref (old_location);
 	}
@@ -1556,64 +1517,20 @@ free_location_change (NautilusWindowSlot *slot)
 			 got_file_info_for_view_selection_callback, slot);
                 slot->details->determine_view_file = NULL;
         }
-
-        if (slot->details->new_content_view != NULL) {
-		g_object_unref (slot->details->new_content_view);
-		slot->details->new_content_view = NULL;
-        }
 }
 
 static void
-cancel_location_change (NautilusWindowSlot *slot)
-{
-	GList *selection;
-	GFile *location;
-        NautilusDirectory *directory;
-
-	location = nautilus_window_slot_get_location (slot);
-
-	directory = nautilus_directory_get (slot->details->location);
-        /* Stops current loading or search if any, so we are not slow */
-        if (NAUTILUS_IS_FILES_VIEW (slot->details->content_view)) {
-                nautilus_files_view_stop_loading (NAUTILUS_FILES_VIEW (slot->details->content_view));
-        }
-        nautilus_directory_unref (directory);
-
-        if (slot->details->pending_location != NULL
-            && location != NULL
-            && slot->details->content_view != NULL
-            && NAUTILUS_IS_FILES_VIEW (slot->details->content_view)) {
-
-                /* No need to tell the new view - either it is the
-                 * same as the old view, in which case it will already
-                 * be told, or it is the very pending change we wish
-                 * to cancel.
-                 */
-		selection = nautilus_view_get_selection (slot->details->content_view);
-                load_new_location (slot,
-				   location,
-				   selection,
-				   TRUE,
-				   FALSE);
-		nautilus_file_list_free (selection);
-        }
-
-        end_location_change (slot);
-}
-
-void
 nautilus_window_slot_set_content_view (NautilusWindowSlot *slot,
-				       const char *id)
+                                       guint                id)
 {
         NautilusFilesView *view;
         GList *selection;
 	char *uri;
 
 	g_assert (slot != NULL);
-	g_assert (id != NULL);
 
 	uri = nautilus_window_slot_get_location_uri (slot);
-	DEBUG ("Change view of window %s to %s", uri, id);
+	DEBUG ("Change view of window %s to %d", uri, id);
 	g_free (uri);
 
 	if (nautilus_window_slot_content_view_matches (slot, id)) {
@@ -1623,7 +1540,7 @@ nautilus_window_slot_set_content_view (NautilusWindowSlot *slot,
         selection = nautilus_view_get_selection (slot->details->content_view);
         view = nautilus_files_view_new (id, slot);
 
-        end_location_change (slot);
+        nautilus_window_slot_stop_loading (slot);
 
         nautilus_window_slot_set_allow_stop (slot, TRUE);
 
@@ -2079,7 +1996,6 @@ view_started_loading (NautilusWindowSlot *slot,
                 nautilus_window_slot_set_allow_stop (slot, TRUE);
         }
 
-        nautilus_query_editor_set_location (slot->details->query_editor, nautilus_view_get_location (view));
         gtk_widget_grab_focus (GTK_WIDGET (slot->details->window));
 
         gtk_widget_show (GTK_WIDGET (slot->details->window));
@@ -2218,7 +2134,8 @@ nautilus_window_slot_switch_new_content_view (NautilusWindowSlot *slot)
 	GtkWidget *widget;
         gboolean reusing_view;
 
-        reusing_view = gtk_widget_get_parent (GTK_WIDGET (slot->details->new_content_view)) != NULL;
+        reusing_view = slot->details->new_content_view &&
+                       gtk_widget_get_parent (GTK_WIDGET (slot->details->new_content_view)) != NULL;
         /* We are either reusing the view, so new_content_view and content_view
          * are the same, or the new_content_view is invalid */
         if (slot->details->new_content_view == NULL || reusing_view)
@@ -2305,11 +2222,6 @@ nautilus_window_slot_dispose (GObject *object)
 
         g_clear_object (&slot->details->location);
 
-        if (slot->details->view_mode_before_search) {
-                g_free (slot->details->view_mode_before_search);
-                slot->details->view_mode_before_search = NULL;
-        }
-
 	nautilus_file_list_free (slot->details->pending_selection);
 	slot->details->pending_selection = NULL;
 
@@ -2534,7 +2446,44 @@ nautilus_window_slot_set_allow_stop (NautilusWindowSlot *slot,
 void
 nautilus_window_slot_stop_loading (NautilusWindowSlot *slot)
 {
-        cancel_location_change (slot);
+        GList *selection;
+        GFile *location;
+        NautilusDirectory *directory;
+
+        location = nautilus_window_slot_get_location (slot);
+
+        directory = nautilus_directory_get (slot->details->location);
+
+        if (NAUTILUS_IS_FILES_VIEW (slot->details->content_view)) {
+                nautilus_files_view_stop_loading (NAUTILUS_FILES_VIEW (slot->details->content_view));
+        }
+
+        nautilus_directory_unref (directory);
+
+        if (slot->details->pending_location != NULL &&
+            location != NULL &&
+            slot->details->content_view != NULL &&
+            NAUTILUS_IS_FILES_VIEW (slot->details->content_view)) {
+                /* No need to tell the new view - either it is the
+                 * same as the old view, in which case it will already
+                 * be told, or it is the very pending change we wish
+                 * to cancel.
+                 */
+                selection = nautilus_view_get_selection (slot->details->content_view);
+                load_new_location (slot,
+                                   location,
+                                   selection,
+                                   TRUE,
+                                   FALSE);
+                nautilus_file_list_free (selection);
+        }
+
+        end_location_change (slot);
+
+        if (slot->details->new_content_view) {
+                g_object_unref (slot->details->new_content_view);
+                slot->details->new_content_view = NULL;
+        }
 }
 
 NautilusView*
