@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 
 /*
  *  Nautilus
@@ -31,7 +31,6 @@
 #include <config.h>
 
 #include "nautilus-application.h"
-#include "nautilus-bookmarks-window.h"
 #include "nautilus-desktop-window.h"
 #include "nautilus-location-entry.h"
 #include "nautilus-mime-actions.h"
@@ -482,9 +481,9 @@ on_location_changed (NautilusWindow *window)
 }
 
 static void
-on_slot_loading_changed (NautilusWindowSlot *slot,
-                         GParamSpec         *pspec,
-                         NautilusWindow     *window)
+on_slot_location_changed (NautilusWindowSlot *slot,
+                          GParamSpec         *pspec,
+                          NautilusWindow     *window)
 {
         if (nautilus_window_get_active_slot (window) == slot)
                 on_location_changed (window);
@@ -514,8 +513,8 @@ static void
 connect_slot (NautilusWindow     *window,
               NautilusWindowSlot *slot)
 {
-        g_signal_connect (slot, "notify::loading",
-                          G_CALLBACK (on_slot_loading_changed), window);
+        g_signal_connect (slot, "notify::location",
+                          G_CALLBACK (on_slot_location_changed), window);
 }
 
 static void
@@ -933,6 +932,95 @@ open_location_cb (NautilusWindow     *window,
                                                  location, flags, NULL, window, NULL);
 }
 
+static void
+notify_unmount_done (GMountOperation *op,
+                     const gchar     *message)
+{
+  GApplication *application;
+  gchar *notification_id;
+
+  application = g_application_get_default ();
+  notification_id = g_strdup_printf ("nautilus-mount-operation-%p", op);
+  g_application_withdraw_notification (application, notification_id);
+
+  if (message != NULL)
+    {
+      GNotification *unplug;
+      GIcon *icon;
+      gchar **strings;
+
+      strings = g_strsplit (message, "\n", 0);
+      icon = g_themed_icon_new ("media-removable");
+      unplug = g_notification_new (strings[0]);
+      g_notification_set_body (unplug, strings[1]);
+      g_notification_set_icon (unplug, icon);
+
+      g_application_send_notification (application, notification_id, unplug);
+      g_object_unref (unplug);
+      g_object_unref (icon);
+      g_strfreev (strings);
+    }
+
+  g_free (notification_id);
+}
+
+static void
+notify_unmount_show (GMountOperation *op,
+                     const gchar     *message)
+{
+  GApplication *application;
+  GNotification *unmount;
+  gchar *notification_id;
+  GIcon *icon;
+  gchar **strings;
+
+  application = g_application_get_default ();
+  strings = g_strsplit (message, "\n", 0);
+  icon = g_themed_icon_new ("media-removable");
+
+  unmount = g_notification_new (strings[0]);
+  g_notification_set_body (unmount, strings[1]);
+  g_notification_set_icon (unmount, icon);
+  g_notification_set_priority (unmount, G_NOTIFICATION_PRIORITY_URGENT);
+
+  notification_id = g_strdup_printf ("nautilus-mount-operation-%p", op);
+  g_application_send_notification (application, notification_id, unmount);
+  g_object_unref (unmount);
+  g_object_unref (icon);
+  g_strfreev (strings);
+  g_free (notification_id);
+}
+
+static void
+show_unmount_progress_cb (GMountOperation *op,
+                          const gchar     *message,
+                          gint64           time_left,
+                          gint64           bytes_left,
+                          gpointer         user_data)
+{
+  if (bytes_left == 0)
+    notify_unmount_done (op, message);
+  else
+    notify_unmount_show (op, message);
+}
+
+static void
+show_unmount_progress_aborted_cb (GMountOperation *op,
+                                  gpointer         user_data)
+{
+  notify_unmount_done (op, NULL);
+}
+
+static void
+places_sidebar_unmount_operation_cb (NautilusWindow  *window,
+                                    GMountOperation  *mount_operation)
+{
+  g_signal_connect (mount_operation, "show-unmount-progress",
+                    G_CALLBACK (show_unmount_progress_cb), NULL);
+  g_signal_connect (mount_operation, "aborted",
+                    G_CALLBACK (show_unmount_progress_aborted_cb), NULL);
+}
+
 /* Callback used when the places sidebar needs us to present an error message */
 static void
 places_sidebar_show_error_message_cb (GtkPlacesSidebar *sidebar,
@@ -946,13 +1034,14 @@ places_sidebar_show_error_message_cb (GtkPlacesSidebar *sidebar,
 }
 
 static void
-places_sidebar_show_other_locations (NautilusWindow   *window)
+places_sidebar_show_other_locations (NautilusWindow   *window,
+                                     GtkPlacesOpenFlags open_flags)
 {
         GFile *location;
 
         location = g_file_new_for_uri ("other-locations:///");
 
-        open_location_cb (window, location, GTK_PLACES_OPEN_NORMAL);
+        open_location_cb (window, location, open_flags);
 
         g_object_unref (location);
 }
@@ -1276,6 +1365,8 @@ nautilus_window_set_up_sidebar (NautilusWindow *window)
 			  G_CALLBACK (places_sidebar_drag_perform_drop_cb), window);
 	g_signal_connect (window->priv->places_sidebar, "populate-popup",
 			  G_CALLBACK (places_sidebar_populate_popup_cb), window);
+	g_signal_connect (window->priv->places_sidebar, "unmount",
+			  G_CALLBACK (places_sidebar_unmount_operation_cb), window);
 }
 
 void
@@ -1488,12 +1579,12 @@ nautilus_window_notification_delete_get_label (NautilusFileUndoInfo *undo_info,
 	if (length == 1) {
 		file_label = g_file_get_basename (files->data);
 	        /* Translators: only one item has been deleted and %s is its name. */
-		label = g_strdup_printf (_("“%s” deleted"), file_label);
+		label = g_markup_printf_escaped (_("“%s” deleted"), file_label);
         	g_free (file_label);
 	} else {
 	        /* Translators: one or more items might have been deleted, and %d
 	         * is the count. */
-		label = g_strdup_printf (ngettext ("%d file deleted", "%d files deleted", length), length);
+		label = g_markup_printf_escaped (ngettext ("%d file deleted", "%d files deleted", length), length);
 	}
 
 	return label;
@@ -2443,6 +2534,9 @@ nautilus_window_init (NautilusWindow *window)
 	window->priv->slots = NULL;
 	window->priv->active_slot = NULL;
 
+	gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (window)),
+				     "nautilus-window");
+
 	window_group = gtk_window_group_new ();
 	gtk_window_group_add_window (window_group, GTK_WINDOW (window));
 	g_object_unref (window_group);
